@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999-2002,2006 Rami Lehti, Pablo Virolainen, Richard
- * van den Berg
+ * Copyright (C) 1999-2002,2006,2011,2015,2016 Rami Lehti, Pablo Virolainen,
+ * Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -22,6 +22,7 @@
 #ifndef _COMMANDCONF_H_INCLUDED
 #define _COMMANDCONF_H_INCLUDED
 #include "list.h"
+#include "gen_list.h"
 #include "db_config.h"
 
 extern long conf_lineno;
@@ -32,7 +33,7 @@ int commandconf(const char mode,const char* line);
 int conf_input_wrapper(char* buf, int max_size, FILE* in);
 int db_input_wrapper(char* buf, int max_size, int db);
 
-list* append_rxlist(char*,DB_ATTR_TYPE,list*);
+list* append_rxlist(char*,DB_ATTR_TYPE,list*, RESTRICTION_TYPE);
 
 void do_define(char*,char*);
 
@@ -44,6 +45,8 @@ int do_ifxhost(int,char*);
 
 void do_groupdef(char*,DB_ATTR_TYPE);
 
+RESTRICTION_TYPE get_restrictionval(char*);
+
 DB_ATTR_TYPE get_groupval(char*);
 
 void putbackvariable(char*);
@@ -58,6 +61,10 @@ void do_replevdef(char*);
 
 void do_repurldef(char*);
 
+void do_rootprefix(char*);
+
+void do_report_ignore_e2fsattrs(char*);
+
 int check_db_order(DB_FIELD*,int, DB_FIELD);
 
 void* get_db_key(void);
