@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999-2002,2004-2006,2010 Rami Lehti, Pablo Virolainen,
- * Richard van den Berg, Hannes von Haugwitz
+ * Copyright (C) 1999-2002,2004-2006,2010-2013,2015,2016 Rami Lehti, Pablo
+ * Virolainen, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -25,11 +25,11 @@
 #include "types.h"
 #include <unistd.h>
 #include <stdio.h>
+#include <pcre.h>
 
 #define E2O(n) (1<<n)
 
 #include "list.h"
-#include "seltree.h"
 
 #ifdef WITH_SUN_ACL /* First try to implement support for sun acl. */
 /*#define WITH_ACL    If we use sun acl then we have acl :) */
@@ -251,6 +251,53 @@ extern const int db_aliasvalue[db_alias_size];
 /*  }_db_config ; */
 
 
+#include "seltree.h"
+
+typedef struct db_line {
+  byte* md5;
+  byte* sha1;
+  byte* rmd160;
+  byte* tiger;
+
+  byte* sha256;
+  byte* sha512;
+
+  byte* crc32; /* MHASH only */
+  byte* haval;
+  byte* gost;
+  byte* crc32b;
+  byte* whirlpool;
+
+  acl_type* acl;
+  /* Something here.. */
+
+  mode_t perm;
+  mode_t perm_o; /* Permission for tree traverse */
+  uid_t uid;
+  gid_t gid;
+  time_t atime;
+  time_t ctime;
+  time_t mtime;
+  AIDE_INO_TYPE inode;
+  nlink_t nlink;
+
+  AIDE_OFF_TYPE size;
+  AIDE_OFF_TYPE size_o; /* ... */
+  AIDE_BLKCNT_TYPE bcount;
+  char* filename;
+  char* fullpath;
+  char* linkname;
+
+  char *cntx;
+
+  xattrs_type* xattrs;
+
+  unsigned long e2fsattrs;
+
+  /* Attributes .... */
+  DB_ATTR_TYPE attr;
+
+} db_line;
 
 typedef struct db_config {
   
@@ -265,6 +312,14 @@ typedef struct db_config {
   
   int config_check;
 
+  struct md_container *mdc_in;
+  struct md_container *mdc_out;
+
+  struct db_line *line_db_in;
+  struct db_line *line_db_out;
+
+  DB_ATTR_TYPE db_attrs;
+
 #ifdef WITH_ZLIB
   gzFile db_gzin;
   gzFile db_gznew;
@@ -309,8 +364,16 @@ typedef struct db_config {
   list* negrxlst;
 
   int verbose_level;
+  int database_add_metadata;
+  int report_detailed_init;
+  int report_base16;
+  int report_quiet;
   int use_initial_errorsto;
 
+#ifdef WITH_E2FSATTRS
+  unsigned long report_ignore_e2fsattrs;
+#endif
+
   url_t* initial_report_url;
   FILE* initial_report_fd;
   
@@ -351,55 +414,16 @@ typedef struct db_config {
 
   int summarize_changes;
 
+  char* root_prefix;
+  int root_prefix_length;
+
+  char* limit;
+  pcre* limit_crx;
+
   struct seltree* tree;
 
 } db_config;
 
-typedef struct db_line {
-  byte* md5;
-  byte* sha1;
-  byte* rmd160;
-  byte* tiger;
-
-  byte* sha256;
-  byte* sha512;
- 
-  byte* crc32; /* MHASH only */
-  byte* haval;
-  byte* gost;
-  byte* crc32b;
-  byte* whirlpool; 
-
-  acl_type* acl;
-  /* Something here.. */
-
-  mode_t perm;
-  mode_t perm_o; /* Permission for tree traverse */
-  uid_t uid;
-  gid_t gid;
-  time_t atime;
-  time_t ctime;
-  time_t mtime;
-  AIDE_INO_TYPE inode;
-  nlink_t nlink;
-  
-  AIDE_OFF_TYPE size;
-  AIDE_OFF_TYPE size_o; /* ... */
-  AIDE_BLKCNT_TYPE bcount;
-  char* filename;
-  char* linkname;
-
-  char *cntx;
- 
-  xattrs_type* xattrs;
-
-  unsigned long e2fsattrs;
-
-  /* Attributes .... */
-  DB_ATTR_TYPE attr;
-  
-} db_line;
-
 #ifdef WITH_PSQL
 #include "libpq-fe.h"
 
