@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999-2006,2010 Rami Lehti, Pablo Virolainen, Mike
- * Markley, Richard van den Berg, Hannes von Haugwitz
+ * Copyright (C) 1999-2006,2010-2013,2015,2016 Rami Lehti, Pablo Virolainen,
+ * Mike Markley, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -28,6 +28,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <sys/types.h>
+#include <dirent.h>
 #include <time.h>
 
 #if HAVE_UNISTD_H
@@ -55,7 +56,7 @@ db_config* conf;
 #define MAXHOSTNAMELEN 256
 #endif
 
-void usage(int exitvalue)
+static void usage(int exitvalue)
 {
   fprintf(stderr, 
 	  _("Aide " AIDEVERSION" \n\n"
@@ -64,13 +65,14 @@ void usage(int exitvalue)
 	    "  -i, --init\t\tInitialize the database\n"
 	    "  -C, --check\t\tCheck the database\n"
 	    "  -u, --update\t\tCheck and update the database non-interactively\n"
-	    "      --compare\t\tCompare two databases\n\n"
+	    "  -E, --compare\t\tCompare two databases\n\n"
 	    "Miscellaneous:\n"
 	    "  -D, --config-check\tTest the configuration file\n"
 	    "  -v, --version\t\tShow version of AIDE and compilation options\n"
 	    "  -h, --help\t\tShow this help message\n\n"
 	    "Options:\n"
 	    "  -c [cfgfile]\t--config=[cfgfile]\tGet config options from [cfgfile]\n"
+	    "  -l [REGEX]\t--limit=[REGEX]\t\tLimit command to entries matching [REGEX]\n"
 	    "  -B \"OPTION\"\t--before=\"OPTION\"\tBefore configuration file is read define OPTION\n"
 	    "  -A \"OPTION\"\t--after=\"OPTION\"\tAfter configuration file is read define OPTION\n"
 	    "  -r [reporter]\t--report=[reporter]\tWrite report output to [reporter] url\n"
@@ -81,7 +83,7 @@ void usage(int exitvalue)
   exit(exitvalue);
 }
 
-void print_version(void)
+static void print_version(void)
 {
   fprintf(stderr,
 	  "Aide " AIDEVERSION "\n\n"
@@ -89,7 +91,7 @@ void print_version(void)
   exit(0);
 }
 
-int read_param(int argc,char**argv)
+static int read_param(int argc,char**argv)
 {
   int option = -1;
   char* err=NULL;
@@ -109,6 +111,7 @@ int read_param(int argc,char**argv)
     { "check", no_argument, NULL, 'C'},
     { "update", no_argument, NULL, 'u'},
     { "config-check", no_argument, NULL, 'D'},
+    { "limit", required_argument, NULL, 'l'},
     { "compare", no_argument, NULL, 'E'},
     { NULL,0,NULL,0 }
   };
@@ -176,7 +179,23 @@ int read_param(int argc,char**argv)
 	}
 	break;
       }
-
+      case 'l': {
+            if (optarg!=NULL) {
+                const char* pcre_error;
+                int pcre_erroffset;
+                conf->limit=malloc(strlen(optarg)+1);
+                strcpy(conf->limit,optarg);
+                if((conf->limit_crx=pcre_compile(conf->limit, PCRE_ANCHORED, &pcre_error, &pcre_erroffset, NULL)) == NULL) {
+                    error(0,_("Error in limit regexp '%s' at %i: %s\n"), conf->limit, pcre_erroffset, pcre_error);
+                    exit(INVALID_ARGUMENT_ERROR);
+                }
+                error(200,_("Limit set to '%s'\n"), conf->limit);
+            } else {
+                error(0,_("-l must have an argument\n"));
+                exit(INVALID_ARGUMENT_ERROR);
+            }
+            break;
+      }
       case 'r': {
 	if(optarg!=NULL) {
 	  do_repurldef(optarg);
@@ -242,12 +261,12 @@ int read_param(int argc,char**argv)
   return RETOK;
 }
 
-void setdefaults_before_config()
+static void setdefaults_before_config()
 {
   char* urlstr=INITIALERRORSTO;
   url_t* u=NULL;
   char* s=(char*)malloc(sizeof(char)*MAXHOSTNAMELEN+1);
-  unsigned long long p;
+  DB_ATTR_TYPE X;
 
   /*
     Set up the hostname
@@ -268,11 +287,18 @@ void setdefaults_before_config()
   conf->tree=NULL;
   conf->config_check=0;
   conf->verbose_level=-1;
+  conf->database_add_metadata=1;
+  conf->report_detailed_init=0;
+  conf->report_base16=0;
+  conf->report_quiet=0;
   conf->use_initial_errorsto=1;
   conf->report_url=NULL;
   conf->report_fd=NULL;
   conf->report_syslog=0;
   conf->report_db=0;
+#ifdef WITH_E2FSATTRS
+  conf->report_ignore_e2fsattrs = 0UL;
+#endif
 
   u=parse_url(urlstr);
   error_init(u,1);
@@ -313,6 +339,23 @@ void setdefaults_before_config()
   conf->db_new=NULL;
   conf->db_out_url=NULL;
   conf->db_out=NULL;
+
+  conf->mdc_in=NULL;
+  conf->mdc_out=NULL;
+
+  conf->line_db_in=NULL;
+  conf->line_db_out=NULL;
+
+  conf->db_attrs = 0;
+#if defined(WITH_MHASH) || defined(WITH_GCRYPT)
+  conf->db_attrs |= DB_MD5|DB_TIGER|DB_HAVAL|DB_CRC32|DB_SHA1|DB_RMD160|DB_SHA256|DB_SHA512;
+#ifdef WITH_MHASH
+  conf->db_attrs |= DB_GOST;
+#ifdef HAVE_MHASH_WHIRLPOOL
+  conf->db_attrs |= DB_WHIRLPOOL;
+#endif
+#endif
+#endif
   
 #ifdef WITH_ZLIB
   conf->db_gzin=0;
@@ -328,7 +371,13 @@ void setdefaults_before_config()
 
   conf->grouped=1;
 
-  conf->summarize_changes=0;
+  conf->summarize_changes=1;
+
+  conf->root_prefix="";
+  conf->root_prefix_length=0;
+
+  conf->limit=NULL;
+  conf->limit_crx=NULL;
 
   conf->selrxlst=NULL;
   conf->equrxlst=NULL;
@@ -384,60 +433,38 @@ void setdefaults_before_config()
   do_groupdef("e2fsattrs",DB_E2FSATTRS);
 #endif
 
-  p=0LLU;
-#if defined(WITH_MHASH) || defined(WITH_GCRYPT)
-  p|=DB_MD5;
-#endif
+  X=0LLU;
 #ifdef WITH_ACL
-  p|=DB_ACL;
+  X|=DB_ACL;
 #endif
 #ifdef WITH_SELINUX
-  p|=DB_SELINUX;
+  X|=DB_SELINUX;
 #endif
 #ifdef WITH_XATTR
-  p|=DB_XATTRS;
+  X|=DB_XATTRS;
 #endif
 #ifdef WITH_E2FSATTRS
-  p|=DB_E2FSATTRS;
+  X|=DB_E2FSATTRS;
 #endif
+
+
   do_groupdef("R",DB_PERM|DB_FTYPE|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_SIZE|
-                  DB_LINKNAME|DB_MTIME|DB_CTIME|p);
+          DB_LINKNAME|DB_MTIME|DB_CTIME
+#if defined(WITH_MHASH) || defined(WITH_GCRYPT)
+          |DB_MD5
+#endif
+          |X);
 
-  p=0LLU;
-#ifdef WITH_ACL
-  p|=DB_ACL;
-#endif
-#ifdef WITH_SELINUX
-  p|=DB_SELINUX;
-#endif
-#ifdef WITH_XATTR
-  p|=DB_XATTRS;
-#endif
-#ifdef WITH_E2FSATTRS
-  p|=DB_E2FSATTRS;
-#endif
-  do_groupdef("L",DB_PERM|DB_FTYPE|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_LINKNAME|p);
+  do_groupdef("L",DB_PERM|DB_FTYPE|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_LINKNAME|X);
 
-  p=0LLU;
-#ifdef WITH_ACL
-  p|=DB_ACL;
-#endif
-#ifdef WITH_SELINUX
-  p|=DB_SELINUX;
-#endif
-#ifdef WITH_XATTR
-  p|=DB_XATTRS;
-#endif
-#ifdef WITH_E2FSATTRS
-  p|=DB_E2FSATTRS;
-#endif
   do_groupdef(">",DB_PERM|DB_FTYPE|DB_INODE|DB_LNKCOUNT|DB_UID|DB_GID|DB_SIZEG|
-		  DB_LINKNAME|p);
+		  DB_LINKNAME|X);
+  do_groupdef("X",X);
   do_groupdef("E",0);
 
 }
 
-void setdefaults_after_config()
+static void setdefaults_after_config()
 {
   if(conf->db_in_url==NULL){
     url_t* u=NULL;
@@ -528,6 +555,20 @@ int main(int argc,char**argv)
 	      "database compare.\n"));
     exit(INVALID_ARGUMENT_ERROR);
   }
+  if (conf->action&(DO_INIT|DO_COMPARE) && conf->root_prefix_length > 0) {
+      DIR *dir;
+      if((dir = opendir(conf->root_prefix)) != NULL) {
+          closedir(dir);
+      } else {
+          char* er=strerror(errno);
+          if (er!=NULL) {
+              error(0,"opendir() for root prefix %s failed: %s\n", conf->root_prefix,er);
+          } else {
+              error(0,"opendir() for root prefix %s failed: %i\n", conf->root_prefix,errno);
+          }
+          exit(INVALID_ARGUMENT_ERROR);
+      }
+  }
 #ifdef WITH_MHASH
   if(conf->config_check&&FORCECONFIGMD){
     error(0,"Can't give config checksum when compiled with --enable-forced_configmd\n");
@@ -560,9 +601,6 @@ int main(int argc,char**argv)
 #endif
   conf->use_initial_errorsto=0;
   if (!conf->config_check) {
-    int exitcode=0;
-    long totalchanges=0;
-    long status[5]={1,0,0,0,0};
     if(conf->action&DO_INIT){
       if(db_init(DB_WRITE)==RETFAIL) {
 	exit(IO_ERROR);
@@ -589,42 +627,9 @@ int main(int argc,char**argv)
     }
       
     populate_tree(conf->tree);
-    db_close(conf);
-
-    /* First terse report */
-    totalchanges=report_tree(conf->tree,0,status);
-    if(totalchanges>0){
-      exitcode=(status[2]!=0)*1+(status[3]!=0)*2+(status[4]!=0)*4;
-      if (conf->grouped) {
-      status[0]=1;
-      report_tree(conf->tree,1,status);
-      status[0]=1;
-      report_tree(conf->tree,2,status);
-      status[0]=1;
-      report_tree(conf->tree,3,status);
-      } else {
-          status[0]=1;
-          report_tree(conf->tree,5,status);
-      }
-      /* Then detailed list of changes */
-      status[0]=1;
-      report_tree(conf->tree,4,status);
-    } else {
-      if (conf->verbose_level >= 5) {
-        printf("\nAIDE, version " AIDEVERSION "\n\n");
-        if(conf->action&DO_COMPARE) {
-          printf("### All files match AIDE database. Looks okay!\n\n");
-        }
-        if(conf->action&DO_INIT) {
-	  if(conf->action&DO_COMPARE)
-            printf("### New AIDE database written to %s\n\n",conf->db_out_url->value);
-	  else
-            printf("### AIDE database at %s initialized.\n\n",conf->db_out_url->value);
-        }
-      }
-    }
+    db_close();
     
-    exit(exitcode);
+    exit(gen_report(conf->tree));
     
   }else {
 #ifdef WITH_MHASH
