@@ -1,8 +1,8 @@
 /* aide, Advanced Intrusion Detection Environment
  * vi: ts=2 sw=2
  *
- * Copyright (C) 1999-2006,2010 Rami Lehti, Pablo Virolainen, Richard
- * van den Berg, Mike Markley, Hannes von Haugwitz
+ * Copyright (C) 1999-2006,2010,2011,2016 Rami Lehti, Pablo Virolainen,
+ * Richard van den Berg, Mike Markley, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -41,7 +41,6 @@
 #include "db_disk.h"
 #include "conf_yacc.h"
 #include "util.h"
-#include "db_sql.h"							/* typedefs */
 #include "commandconf.h"
 /*for locale support*/
 #include "locale-aide.h"
@@ -62,19 +61,21 @@ static struct AIDE_DIRENT_TYPE **resp = NULL;
 static struct seltree *r = NULL;
 
 
-static const char *dot = ".";
-static const char *dotdot = "..";
-
 #if defined HAVE_READDIR && !defined HAVE_READDIR_R
 static long td = -1;
 #endif
 static int rdres = 0;
-static DB_ATTR_TYPE attr;
-static char *start_path = "/";
 
 static int root_handled = 0;
 
-static int open_dir (void);
+static DIR *open_dir(char* path) {
+   if (dirh != NULL) {
+       if (closedir(dirh) != 0) {
+           /* Closedir did not success? */
+       }
+   }
+   return opendir(path);
+}
 
 static void next_in_dir (void)
 {
@@ -110,7 +111,7 @@ static char *name_construct (const char *s)
 {
 	char *ret;
 	int len2 = strlen (r->path);
-	int len = len2 + strlen (s) + 2;
+	int len = len2 + strlen (s) + 2 + conf->root_prefix_length;
 
 	if (r->path[len2 - 1] != '/') {
 		len++;
@@ -118,9 +119,10 @@ static char *name_construct (const char *s)
 
 	ret = (char *) malloc (len);
 	ret[0] = (char) 0;
-	strcpy (ret, r->path);
+	strncpy(ret, conf->root_prefix, conf->root_prefix_length+1);
+	strncat (ret, r->path, len2);
 	if (r->path[len2 - 1] != '/') {
-		strcat (ret, "/");
+		strncat (ret, "/", 1);
 	}
 	strcat (ret, s);
 	return ret;
@@ -150,7 +152,7 @@ void add_child (db_line * fil)
 	i = strlen (fil->filename);
 
 	new_r->path = malloc (i + 1);
-	strcpy (new_r->path, fil->filename);
+	strncpy(new_r->path, fil->filename, i+1);
 	new_r->childs = NULL;
 	new_r->sel_rx_lst = NULL;
 	new_r->neg_rx_lst = NULL;
@@ -168,46 +170,56 @@ void add_child (db_line * fil)
 	r->childs = list_sorted_insert (r->childs, new_r, compare_node_by_path);
 }
 
+static int get_file_status(char *filename, struct AIDE_STAT_TYPE *fs) {
+    int sres = 0;
+    sres = AIDE_LSTAT_FUNC(filename,fs);
+    if(sres == -1){
+        char* er = strerror(errno);
+        if (er == NULL) {
+            error(0,"get_file_status: lstat() failed for %s. strerror() failed for %i\n", filename, errno);
+        } else {
+            error(0,"get_file_status: lstat() failed for %s: %s\n", filename, er);
+        }
+    }
+    return sres;
+}
+
 /*
   It might be a good idea to make this non recursive.
   Now implemented with goto-statement. Yeah, it's ugly and easy.
 */
 
-db_line *db_readline_disk (int db)
+db_line *db_readline_disk ()
 {
 	db_line *fil = NULL;
+	DB_ATTR_TYPE attr;
 	char *fullname;
 	int add = 0;
+	struct AIDE_STAT_TYPE fs;
 
 	/* root needs special handling */
 	if (!root_handled) {
-		fullname = malloc (1 + 1);
-		strcpy (fullname, "/");
-		add = check_rxtree (fullname, conf->tree, &attr);
-		error (240, "%s match=%d, tree=%p, attr=%llu\n", fullname, add,
+		root_handled = 1;
+		fullname=malloc((conf->root_prefix_length+2)*sizeof(char));
+		strncpy(fullname, conf->root_prefix, conf->root_prefix_length+1);
+		strncat (fullname, "/", 1);
+		if (!get_file_status(&fullname[conf->root_prefix_length], &fs)) {
+		add = check_rxtree (&fullname[conf->root_prefix_length], conf->tree, &attr, fs.st_mode);
+		error (240, "%s match=%d, tree=%p, attr=%llu\n", &fullname[conf->root_prefix_length], add,
 					 conf->tree, attr);
 
-		if (add) {
-			fil = get_file_attrs (fullname, attr);
+		if (add > 0) {
+			fil = get_file_attrs (fullname, attr, &fs);
+
+			error (240, "%s attr=%llu\n", &fullname[conf->root_prefix_length], attr);
 
-			error (240, "%s attr=%llu\n", fullname, attr);
 			if (fil != NULL) {
 				error (240, "%s attr=%llu\n", fil->filename, fil->attr);
+				return fil;
 			}
-
-			if (fil == NULL) {
-				/*
-				   Something went wrong during read process -> 
-				   Let's try next one.
-				 */
-				free_db_line (fil);			/* Filename is freeed? */
-				free (fil);
-				fil = NULL;
-			}
+        }
 		}
-		if (!add)
-			free (fullname);
-		root_handled = 1;
+		free (fullname);
 	}
 recursion:
 	next_in_dir ();
@@ -219,7 +231,7 @@ recursion:
 		   If have, just skipit.
 		   If don't do the 'normal' thing.
 		 */
-		if (strcmp (entp->d_name, dot) == 0 || strcmp (entp->d_name, dotdot) == 0) {
+		if (strcmp (entp->d_name, ".") == 0 || strcmp (entp->d_name, "..") == 0) {
 			goto recursion;						// return db_readline_disk(db);
 		}
 
@@ -237,28 +249,27 @@ recursion:
 		   If not call, db_readline_disk again...
 		 */
 
-		add = check_rxtree (fullname, conf->tree, &attr);
-		error (240, "%s match=%d, tree=%p, attr=%llu\n", fullname, add,
+		if (get_file_status(&fullname[conf->root_prefix_length], &fs)) {
+		    free (fullname);
+		    goto recursion;
+		}
+		add = check_rxtree (&fullname[conf->root_prefix_length], conf->tree, &attr, fs.st_mode);
+		error (240, "%s match=%d, tree=%p, attr=%llu\n", &fullname[conf->root_prefix_length], add,
 					 conf->tree, attr);
 
-		if (add) {
-			fil = get_file_attrs (fullname, attr);
+		if (add > 0) {
+			fil = get_file_attrs (fullname, attr, &fs);
+
+			error (240, "%s attr=%llu\n", &fullname[conf->root_prefix_length], attr);
 
-			error (240, "%s attr=%llu\n", fullname, attr);
 			if (fil != NULL) {
 				error (240, "%s attr=%llu\n", fil->filename, fil->attr);
-			}
-			/*
-			   Hack.
-			 */
-
-			if (fil == NULL) {
+			} else {
 				/*
 				   Something went wrong during read process -> 
 				   Let's try next one.
 				 */
-				free_db_line (fil);			/* Filename is freeed? */
-				fil = NULL;
+				free (fullname);
 				goto recursion;					// return db_readline_disk(db);
 			}
 
@@ -328,14 +339,12 @@ recursion:
 
 				error (255, "r->childs %p, r->parent %p,r->checked %i\n",
 							 r->childs, r->parent, r->checked);
-				/*
-				   Hack.
-				 */
-				start_path = r->path;
+				fullname=malloc((conf->root_prefix_length+strlen(r->path)+1)*sizeof(char));
+				strncpy(fullname, conf->root_prefix, conf->root_prefix_length+1);
+				strncat(fullname, r->path, strlen(r->path));
+				dirh=open_dir(fullname);
+				if (! dirh) {
 
-				error (255, "New start_path=%s\n", start_path);
-
-				if (open_dir () == RETFAIL) {
 					/* open_dir failed so we need to know why and print 
 					   an errormessage if needed.
 					   errno should still be the one from opendir() since it's global
@@ -358,20 +367,21 @@ recursion:
 						   having rules referring to them.
 						 */
 						error (10,
-									 "There are rules referring to non-existent directory %s\n", start_path);
+									 "There are rules referring to non-existent directory %s\n", fullname);
 					} else if (errno != ENOTDIR) {
 						/* We print the message unless it is "Not a directory". */
 						char *er = strerror (errno);
 						if (er != NULL) {
-							error (3, "open_dir():%s: %s\n", er, start_path);
+							error (3, "open_dir(): %s: %s\n", er, fullname);
 						} else {
-							error (3, "open_dir():%i: %s\n", errno, start_path);
+							error (3, "open_dir(): %i: %s\n", errno, fullname);
 						}
 					}
 					r->checked |= NODE_TRAVERSE | NODE_CHECKED;
 					r = r->parent;
 					error (255, "dropping back to parent\n");
 				}
+				free(fullname);
 			} else {
 				r->checked |= NODE_TRAVERSE | NODE_CHECKED;
 				r = r->parent;
@@ -402,31 +412,6 @@ recursion:
 	return fil;
 }
 
-static int open_dir (void)
-{
-	if (dirh != NULL) {
-		if (closedir (dirh) != 0) {
-			/*
-			   Closedir did not success?
-			 */
-		}
-
-	}
-
-	dirh = opendir (start_path);
-	if (dirh == NULL) {
-		/* Errors should not be printed here because then we get too many
-		   error messages. */
-		return RETFAIL;
-	}
-
-	/*
-	   Init the first time.
-	 */
-	return RETOK;
-
-}
-
 int db_disk_init ()
 {
 
@@ -448,34 +433,21 @@ int db_disk_init ()
 #   endif
 #  endif
 
-	open_dir ();
 
-	return RETOK;
-}
+	char* fullname=malloc((conf->root_prefix_length+2)*sizeof(char));
+	strncpy(fullname, conf->root_prefix, conf->root_prefix_length+1);
+	strncat (fullname, "/", 1);
+	dirh=open_dir(fullname);
+	free(fullname);
 
-int db_disk_read_spec (int db)
-{
 	return RETOK;
 }
 
 /*
-  We don't support writing to disk, since we are'n a backup/restore software
+  We don't support writing to the pseudo-database disk, since we are'n a
+  backup/restore software. Hence the functions db_writespec_disk,
+  db_writeline_disk and db_close_disk aren't declared.
  */
 
-int db_writespec_disk (db_config * dbconf)
-{
-	return RETFAIL;
-}
-
-int db_writeline_disk (db_line * line, db_config * dbconf)
-{
-	return RETFAIL;
-}
-
-int db_close_disk (db_config * dbconf)
-{
-	return RETOK;
-}
-
 const char *aide_key_6 = CONFHMACKEY_06;
 const char *db_key_6 = DBHMACKEY_06;
