@@ -1,6 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999,2000,2001,2002 Rami Lehti, Pablo Virolainen
+ * Copyright (C) 1999-2002,2004-2006,2010,2011,2013,2016 Rami Lehti, Pablo
+ * Virolainen, Mike Markley, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -109,8 +110,7 @@ url_t* parse_url(char* val)
       u->value=strdup(r);
       r[0]='\0';
       if(gethostname(hostname,MAXHOSTNAMELEN)==-1){
-	free(hostname);
-	hostname="localhost";
+    strncpy(hostname,"localhost", 10);
       }
       if( (strcmp(t,"localhost")==0)||(strcmp(t,hostname)==0)){
 	free(hostname);
@@ -305,7 +305,7 @@ char* perm_to_char(mode_t perm)
   }
 #endif
 
-  error(230,"perm_to_char(): %i -> %s\n",perm,pc);
+  error(240,"perm_to_char(): %i -> %s\n",perm,pc);
 
   return pc;
 }
@@ -361,6 +361,32 @@ void sig_handler(int signum)
   return;
 }
 
+char *expand_tilde(char *path) {
+    char *homedir, *full;
+    size_t path_len, homedir_len, full_len;
+
+    if (path != NULL) {
+        if (path[0] == '~') {
+            if((homedir=getenv("HOME")) != NULL) {
+                path_len = strlen(path+sizeof(char));
+                homedir_len = strlen(homedir);
+                full_len = homedir_len+path_len;
+                full = malloc(sizeof(char) * (full_len+1));
+                strncpy(full, homedir, homedir_len);
+                strncpy(full+homedir_len, path+sizeof(char), path_len);
+                full[full_len] = '\0';
+                free(path);
+                /* Don't free(homedir); because it is not safe on some platforms */
+                path = full;
+            } else {
+                error(3, _("Variable name 'HOME' not found in environment. '~' cannot be expanded\n"));
+            }
+        } else if (path[0] == '\\' && path[1] == '~') {
+            path += sizeof(char);
+        }
+    }
+    return path;
+}
 
 /* Like strstr but only do search for maximum of n chars.
    haystack does not have to be NULL terminated
